#include <QNetworkReply>
#include <QNetworkAccessManager>
#include <QUrl>
#include <QStringList>
#include <QNetworkCookie>
#include <QRegExp>
#include <QVariant>
#include <QtDebug>

#include "googlereader.h"

GoogleReader::GoogleReader(QObject *parent) :
    QObject(parent),
    m_connected(false),
    m_activeJob(false),
    m_mail(),
    m_pass(),
    m_sid()
{
    qDebug() << "in init";
    m_manager = new QNetworkAccessManager();
    m_subscriptionList = new Subscriptions(this);
}

void GoogleReader::getSID()
{
    qDebug() << "request sid...";

    m_isConnecting = true;

    QUrl url;
    url.setUrl("https://www.google.com/accounts/ClientLogin");
    url.addQueryItem("service", "reader");
    url.addQueryItem("Email", m_mail);
    url.addQueryItem("Passwd", m_pass);
    url.addQueryItem("source", "Akonadi Google-Reader RSS resource");
    url.addQueryItem("continue", "http://www.google.com/");

    QNetworkRequest loginRequest;
    loginRequest.setUrl(url);
    loginRequest.setRawHeader("User-Agent", "Google-Reader Akonadi News fetcher");

    m_loginReply = m_manager->post(loginRequest, url.encodedQuery());

    connect(m_loginReply, SIGNAL(finished()), this, SLOT(sidReply()));
    connect(m_loginReply, SIGNAL(error(QNetworkReply::NetworkError)), this, SLOT(error(QNetworkReply::NetworkError)));
    connect(m_loginReply, SIGNAL(sslErrors(const QList<QSslError>&)), this, SLOT(sslErrors(const QList<QSslError>&)));
}

void GoogleReader::sidReply()
{
    qDebug() << "ssid reply!";
    //qDebug() << m_loginReply->readAll();
    QList<QByteArray> data = m_loginReply->readAll().split('\n');

    for (int i = 0; i < data.size(); ++i) {
        if (data.at(i).startsWith("SID="))
        {
            qDebug() << "Found SID:" << data.at(i).mid(4);
            m_sid = data.at(i).mid(4);
        }
        else if (data.at(i).startsWith("Auth="))
        {
            qDebug() << "Found Auth:" << data.at(i).mid(5);
            m_auth = data.at(i).mid(5);
        }

     }

    getToken();
}

QVariant GoogleReader::_getCookie()
{
    if (!m_connected)
        return QVariant();

    QNetworkCookie cookie;
    cookie.setName(QByteArray("SID"));
    cookie.setValue(m_sid);
    cookie.setDomain(QByteArray(".google.com"));
    cookie.setPath(QByteArray("/"));

    QList<QNetworkCookie> cookies;
    cookies.append(cookie);

    return qVariantFromValue(cookies);
}

void GoogleReader::getToken()
{
    qDebug() << "request token...";

    QNetworkRequest tokenRequest;
    tokenRequest.setUrl(QUrl("http://www.google.com/reader/api/0/token"));
    tokenRequest.setRawHeader("User-Agent", "Akonadi Google-Reader RSS resource");
    tokenRequest.setRawHeader("Authorization", "GoogleLogin auth=" + m_auth);
    tokenRequest.setHeader(QNetworkRequest::CookieHeader, _getCookie());

    m_tokenReply = m_manager->get(tokenRequest);

    connect(m_tokenReply, SIGNAL(finished()), this, SLOT(tokenReply()));
    connect(m_tokenReply, SIGNAL(error(QNetworkReply::NetworkError)), this, SLOT(error(QNetworkReply::NetworkError)));
    connect(m_tokenReply, SIGNAL(sslErrors(const QList<QSslError>&)), this, SLOT(sslErrors(const QList<QSslError>&)));

}

void GoogleReader::tokenReply()
{
    qDebug() << "token reply!";
    qDebug() << m_tokenReply->readAll();

    m_isConnecting = false;
    m_connected = true;

    checkJobs();
}

void GoogleReader::fetchFeeds()
{
    addJob(GoogleReader::FetchFeeds);
}

void GoogleReader::_fetchFeeds()
{
    qDebug() << "fetch feeds...";

    QNetworkRequest fetchRequest;
    fetchRequest.setUrl(QUrl("http://www.google.com/reader/api/0/subscription/list"));
    fetchRequest.setRawHeader("User-Agent", "Akonadi Google-Reader RSS resource");
    fetchRequest.setRawHeader("Authorization", "GoogleLogin auth=" + m_auth);
    fetchRequest.setHeader(QNetworkRequest::CookieHeader, _getCookie());

    m_fetchFeedsReply = m_manager->get(fetchRequest);

    connect(m_fetchFeedsReply, SIGNAL(finished()), this, SLOT(fetchFeedsReply()));
    connect(m_fetchFeedsReply, SIGNAL(error(QNetworkReply::NetworkError)), this, SLOT(error(QNetworkReply::NetworkError)));
    connect(m_fetchFeedsReply, SIGNAL(sslErrors(const QList<QSslError>&)), this, SLOT(sslErrors(const QList<QSslError>&)));
}

void GoogleReader::fetchFeedsReply()
{
    qDebug() << "fetch reply!";
    QByteArray xml = m_fetchFeedsReply->readAll();
    m_subscriptionList->read(xml.constData());

    emit subscriptionsUpdated(m_subscriptionList);

    m_activeJob = false;
    checkJobs();
}


void GoogleReader::fetchFeed(QString id)
{
    addJob(GoogleReader::FetchFeed, QVariant(id));
}

void GoogleReader::_fetchFeed(QString id, QDateTime timestamp, QChar sort, int max)
{

    qDebug() << "fetch feed " << id << "...";

    QUrl url = QUrl("http://www.google.com/reader/api/0/stream/contents/" + id);
    if (timestamp.isValid())
        url.addQueryItem("ot", QString(timestamp.toTime_t())); // The time from which you want to retrieve items
    url.addQueryItem("r", QString(sort)); // Sort order of item results. d or n gives items in descending date order, o in ascending order
    url.addQueryItem("xt", "user/-/state/com.google/read"); // e.g. only read items
    url.addQueryItem("n", QString::number(max)); // The maximum number of results to return
    url.addQueryItem("ck", QString(QDateTime::currentDateTime().toTime_t())); // current Unix time here, helps Google with caching
    url.addQueryItem("client", "Akonadi Google-Reader RSS resource");

    QNetworkRequest fetchRequest;
    fetchRequest.setUrl(url);
    fetchRequest.setRawHeader("User-Agent", "Akonadi Google-Reader RSS resource");
    fetchRequest.setRawHeader("Authorization", "GoogleLogin auth=" + m_auth);
    fetchRequest.setHeader(QNetworkRequest::CookieHeader, _getCookie());

    m_fetchFeedReply = m_manager->get(fetchRequest);

    connect(m_fetchFeedReply, SIGNAL(finished()), this, SLOT(fetchFeedReply()));
    connect(m_fetchFeedReply, SIGNAL(error(QNetworkReply::NetworkError)), this, SLOT(error(QNetworkReply::NetworkError)));
    connect(m_fetchFeedReply, SIGNAL(sslErrors(const QList<QSslError>&)), this, SLOT(sslErrors(const QList<QSslError>&)));

}

void GoogleReader::fetchFeedReply()
{

    qDebug() << "fetch reply!";
    QString id = m_fetchFeedReply->request().url().toString().split("http://www.google.com/reader/api/0/stream/contents/").at(1);
    QString cont = m_fetchFeedReply->readAll();

    /* Use QJson for content? */


    m_activeJob = false;
    checkJobs();
}

void GoogleReader::addJob(GoogleReader::Job job, QVariant value)
{
    QPair<GoogleReader::Job, QVariant> pair(job, value);
    m_pendingJobs.append(pair);

    checkJobs();
}

void GoogleReader::checkJobs()
{
    if (m_pendingJobs.empty())
        return;


    if(!m_connected && !m_isConnecting)
    {
        getSID();
        return;
    }
    else if (!m_connected)
    {
        // We get connected...
        return;
    }

    // We are connected to the Google Reader :-)

    if (m_activeJob)
        return;

    m_activeJob = true;
    QPair<GoogleReader::Job, QVariant> pair = m_pendingJobs.takeFirst();
    GoogleReader::Job job = pair.first;
    QVariant value = pair.second;

    if (job == GoogleReader::FetchFeeds)
    {
        _fetchFeeds();
    }
    else if (job == GoogleReader::FetchFeed)
    {
        _fetchFeed(value.toString());
    }

}

void GoogleReader::error(QNetworkReply::NetworkError error)
{
    qDebug() << "error!" << error;
}

void GoogleReader::sslErrors(const QList<QSslError> &errors)
{
    qDebug() << "ssl error!";
}

