/*
    Copyright (C) 2009    Dmitry Ivanov <vonami@gmail.com>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "googlereaderjobs.h"

#include <krssresource/util.h>
#include <krss/rssitem.h>
#include <krss/item.h>
#include <krss/person.h>
#include <krss/category.h>

#include <KIO/Job>
#include <KLocale>
#include <KDebug>

#include <QVariantMap>
#include <QMap>

#include <qjson/parser.h>
#include <KDateTime>

using namespace KRss;
using namespace KRssResource;

static const char separator = '@';

GooglereaderFeedsRetrieveJob::GooglereaderFeedsRetrieveJob( GooglereaderConnection *connection, QObject *parent )
    : FeedsRetrieveJob( parent )
{
    m_connection = connection;
}

QList<Akonadi::Collection> GooglereaderFeedsRetrieveJob::feeds() const
{
    return m_feeds;
}

QString GooglereaderFeedsRetrieveJob::errorString() const
{
    QString result;
    switch ( error() ) {
        case KJob::NoError:
            result = i18n( "No error." );
            break;
        case GooglereaderFeedsRetrieveJob::CouldNotContactBackend:
            result = i18n( "Couldn't contact the Googlereader service.\n%1", errorText() );
            break;
        case GooglereaderFeedsRetrieveJob::CouldNotRetrieveFeeds:
            result = i18n( "Couldn't retrieve feeds.\n%1", errorText() );
            break;
        default:
            result = i18n( "Unknown result code." );
            break;
    }
    return result;
}

void GooglereaderFeedsRetrieveJob::start()
{
    kWarning() << "starting GooglereaderFeedsRetrieveJob!";
    connect(m_connection, SIGNAL(loggedIn()), this, SLOT(loggedIn()));
    m_connection->login();
}

void GooglereaderFeedsRetrieveJob::loggedIn()
{
    kWarning() << "logged in!";
    m_data.clear();

    KIO::TransferJob* job = KIO::get( KUrl( "http://www.google.com/reader/api/0/subscription/list?output=json" ), KIO::Reload, KIO::HideProgressInfo );
    job->addMetaData(QLatin1String("UserAgent"), QLatin1String("Google-Reader Akonadi News fetcher"));
    job->addMetaData(QLatin1String("customHTTPHeader"), QLatin1String("Authorization: GoogleLogin auth="+ m_connection->authorization()));
    
    connect( job, SIGNAL( data( KIO::Job*, const QByteArray& ) ),
             this, SLOT( slotData( KIO::Job*, const QByteArray& ) ) );
    connect( job, SIGNAL( result( KJob* ) ), this, SLOT( slotResult( KJob* ) ) );
    job->start();
}

void GooglereaderFeedsRetrieveJob::slotData( KIO::Job *job, const QByteArray& data )
{
    Q_UNUSED( job )
    kWarning() << "Got data:" << data;
    m_data.append(data);
}

void GooglereaderFeedsRetrieveJob::slotResult( KJob *job )
{
    if ( job->error() ) {
        kWarning() << job->errorString();
        setError( CouldNotRetrieveFeeds );
        setErrorText( job->errorString() );
        emitResult();
        return;
    }

    QJson::Parser parser;
    bool ok;
    kWarning() << "data " << m_data;
    QVariantMap result = parser.parse(m_data, &ok).toMap();
    //kWarning() << "result" << result;

    kWarning() << "ok?" << ok;

    kWarning() << "Subscriptions:";// << result[QLatin1String("subscriptions")].toMap();

    QMap<QString, Akonadi::Collection> storedCategories;

    Akonadi::Collection loc;
    loc.setRemoteId( m_connection->username() );
    loc.setName( loc.remoteId() );
    loc.setParent( Akonadi::Collection::root() );
    loc.setContentMimeTypes( QStringList( Akonadi::Collection::mimeType() ) );
    m_feeds.append( loc );

    foreach (QVariant subscription, result[QLatin1String("subscriptions")].toList()) {
      QVariantMap map = subscription.toMap();
      kWarning() << "\t-" << map[QLatin1String("id")];

      FeedCollection feed;
      feed.setTitle( map[QLatin1String("title")].toString() );
      feed.setRemoteId( map[QLatin1String("id")].toString() );
      feed.setName( feed.title() + QLatin1Char( '-' ) + feed.remoteId() );
      feed.setXmlUrl( QLatin1String("http://www.google.com/reader/api/0/stream/contents/") + map[QLatin1String("id")].toString());

      // Use the first category as parent
      QVariantMap categories = map[QLatin1String("categories")].toList().at(0).toMap();
      //kWarning() << categories[QLatin1String("label")];

      QString id = categories[QLatin1String("id")].toString();
      Akonadi::Collection feedCat;
      /*if (storedCategories.contains(id))
        feedCat = storedCategories.value(id);
      else
      {
        Akonadi::Collection cat;
        cat.setRemoteId( id );
        cat.setName( categories[QLatin1String("label")].toString() );
        cat.setParent( loc );
        cat.setContentMimeTypes( QStringList( Akonadi::Collection::mimeType() ) );
        storedCategories[id] = cat;
        m_feeds.append(cat);
        feedCat = cat;
      }*/

      feedCat = loc;

      feed.setParent( feedCat );
      feed.setContentMimeTypes( QStringList( QLatin1String("application/rss+xml") ) );
      m_feeds.append( feed );
      //kWarning() << "\t-" << subscription[QLatin1String("id")].toString() << subscription[QLatin1String("title")].toString();
    }
    
    emitResult();
}



GooglereaderFeedCreateJob::GooglereaderFeedCreateJob( GooglereaderConnection *connection, QObject *parent )
    : FeedCreateJob( parent ), m_subscriptionId( 0 )
{
    m_connection = connection;
}

Akonadi::Collection GooglereaderFeedCreateJob::feed() const
{
    return m_feed;
}

void GooglereaderFeedCreateJob::setXmlUrl( const QString& xmlUrl )
{
    m_xmlUrl = xmlUrl;
}

QString GooglereaderFeedCreateJob::errorString() const
{
    QString result;
    switch ( error() ) {
        case KJob::NoError:
            result = i18n( "No error." );
            break;
        case GooglereaderFeedCreateJob::CouldNotContactBackend:
            result = i18n( "Couldn't contact the Googlereader service.\n%1", errorText() );
            break;
        case GooglereaderFeedCreateJob::CouldNotCreateFeed:
            result = i18n( "Couldn't create feed.\n%1", errorText() );
            break;
        default:
            result = i18n( "Unknown result code." );
            break;
    }
    return result;
}

void GooglereaderFeedCreateJob::start()
{
    connect(m_connection, SIGNAL(loggedIn()), this, SLOT(loggedIn()));
    m_connection->login();
}

void GooglereaderFeedCreateJob::loggedIn()
{
    KUrl url( "http://www.google.com/reader/api/0/subscription/edit" );
    url.addQueryItem(QLatin1String("s"), QLatin1String("feed/") + m_feed.xmlUrl());
    url.addQueryItem(QLatin1String("ac"), QLatin1String("subscribe"));
    url.addQueryItem(QLatin1String("t"), m_feed.title());
    url.addQueryItem(QLatin1String("T"), m_connection->token());
    //url.addQueryItem(QLatin1String("a"), QString(m_feed.tags()[0].toEncoded()));
    
    KIO::TransferJob* job = KIO::storedHttpPost( url.encodedQuery(), url, KIO::HideProgressInfo );
    job->addMetaData(QLatin1String("UserAgent"), QLatin1String("Google-Reader Akonadi News fetcher"));
    job->addMetaData(QLatin1String("customHTTPHeader"), QLatin1String("Authorization: GoogleLogin auth="+ m_connection->authorization()));

    connect( job, SIGNAL( data( KIO::Job*, const QByteArray& ) ),
             this, SLOT( slotData( KIO::Job*, const QByteArray& ) ) );
    connect( job, SIGNAL( result( KJob* ) ), this, SLOT( slotResult( KJob* ) ) );
    job->start();
}

void GooglereaderFeedCreateJob::slotResult( KJob *job )
{
    if ( job->error() ) {
        kWarning() << job->errorString();
        setError( CouldNotCreateFeed );
        setErrorText( job->errorString() );
        emitResult();
        return;
    }
}

void GooglereaderFeedCreateJob::slotFeedsRetrieved( KJob *job )
{
    if ( job->error() ) {
        kWarning() << job->errorString();
        setError( CouldNotCreateFeed );
        setErrorText( i18n( "Error occured while retrieving the subscription list: %1",
                      job->errorString() ) );
        emitResult();
        return;
    }

    const GooglereaderFeedsRetrieveJob * const rjob = qobject_cast<const GooglereaderFeedsRetrieveJob*>( job );
    Q_ASSERT( rjob );
    const QList<Akonadi::Collection> cols = rjob->feeds();
    Q_FOREACH( const Akonadi::Collection& col, cols ) {
        if ( col.remoteId() == QString::number( m_subscriptionId ) ) {
            kDebug() << "Found the created subscription:" << col.name();
            kDebug() << "Parent id:" << col.parent();
            kDebug() << "Parent remote id:" << col.parentRemoteId();
            m_feed = col;
            emitResult();
            return;
        }
    }

    setError( CouldNotCreateFeed );
    setErrorText( i18n( "Couldn't find the created feed with id %1 in the subscription list",
                  m_subscriptionId ) );
    kWarning() << errorString();
    emitResult();
}



GooglereaderFeedModifyJob::GooglereaderFeedModifyJob( GooglereaderConnection *connection, QObject *parent )
    : FeedModifyJob( parent )
{
    m_connection = connection;
}

void GooglereaderFeedModifyJob::setFeed( const Akonadi::Collection& feed )
{
    m_feed = feed;
}

Akonadi::Collection GooglereaderFeedModifyJob::feed() const
{
    return m_feed;
}

QString GooglereaderFeedModifyJob::errorString() const
{
    QString result;
    switch ( error() ) {
        case KJob::NoError:
            result = i18n( "No error." );
            break;
        case GooglereaderFeedModifyJob::CouldNotContactBackend:
            result = i18n( "Couldn't contact the Googlereader service.\n%1", errorText() );
            break;
        case GooglereaderFeedModifyJob::CouldNotModifyFeed:
            result = i18n( "Couldn't modify feed.\n%1", errorText() );
            break;
        default:
            result = i18n( "Unknown result code." );
            break;
    }
    return result;
}

void GooglereaderFeedModifyJob::start()
{
    connect(m_connection, SIGNAL(loggedIn()), this, SLOT(loggedIn()));
    m_connection->login();
}

void GooglereaderFeedModifyJob::loggedIn()
{
    KUrl url( "http://www.google.com/reader/api/0/subscription/edit" );
    url.addQueryItem(QLatin1String("s"), QLatin1String("feed/") + m_feed.xmlUrl());
    url.addQueryItem(QLatin1String("ac"), QLatin1String("edit"));
    url.addQueryItem(QLatin1String("t"), m_feed.title());
    url.addQueryItem(QLatin1String("T"), m_connection->token());
    //url.addQueryItem(QLatin1String("a"), QString(m_feed.tags()[0].toEncoded()));

    KIO::TransferJob* job = KIO::storedHttpPost( url.encodedQuery(), url, KIO::HideProgressInfo );
    job->addMetaData(QLatin1String("UserAgent"), QLatin1String("Google-Reader Akonadi News fetcher"));
    job->addMetaData(QLatin1String("customHTTPHeader"), QLatin1String("Authorization: GoogleLogin auth="+ m_connection->authorization()));

    connect( job, SIGNAL( data( KIO::Job*, const QByteArray& ) ),
             this, SLOT( slotData( KIO::Job*, const QByteArray& ) ) );
    connect( job, SIGNAL( result( KJob* ) ), this, SLOT( slotResult( KJob* ) ) );
    job->start();
}

void GooglereaderFeedModifyJob::slotResult( KJob *job )
{
    if ( job->error() ) {
        kWarning() << job->errorString();
        setError( CouldNotModifyFeed );
        setErrorText( job->errorString() );
        emitResult();
        return;
    }

    emitResult();
}



GooglereaderFeedDeleteJob::GooglereaderFeedDeleteJob( GooglereaderConnection *connection, QObject *parent )
    : FeedDeleteJob( parent )
{
    m_connection = connection;
}

void GooglereaderFeedDeleteJob::setFeed( const Akonadi::Collection& feed )
{
    m_feed = feed;
}

Akonadi::Collection GooglereaderFeedDeleteJob::feed() const
{
    return m_feed;
}

QString GooglereaderFeedDeleteJob::errorString() const
{
    QString result;
    switch ( error() ) {
        case KJob::NoError:
            result = i18n( "No error." );
            break;
        case GooglereaderFeedDeleteJob::CouldNotContactBackend:
            result = i18n( "Couldn't contact the Googlereader service.\n%1", errorText() );
            break;
        case GooglereaderFeedDeleteJob::CouldNotDeleteFeed:
            result = i18n( "Couldn't delete feed.\n%1", errorText() );
            break;
        default:
            result = i18n( "Unknown result code." );
            break;
    }
    return result;
}

void GooglereaderFeedDeleteJob::start()
{
    connect(m_connection, SIGNAL(loggedIn()), this, SLOT(loggedIn()));
    m_connection->login();
}

void GooglereaderFeedDeleteJob::loggedIn()
{
    KUrl url( "http://www.google.com/reader/api/0/subscription/edit" );
    url.addQueryItem(QLatin1String("s"), QLatin1String("feed/") + m_feed.xmlUrl());
    url.addQueryItem(QLatin1String("ac"), QLatin1String("unsubscribe"));
    url.addQueryItem(QLatin1String("T"), m_connection->token());
    //url.addQueryItem(QLatin1String("a"), QString(m_feed.tags()[0].toEncoded()));

    KIO::TransferJob* job = KIO::storedHttpPost( url.encodedQuery(), url, KIO::HideProgressInfo );
    job->addMetaData(QLatin1String("UserAgent"), QLatin1String("Google-Reader Akonadi News fetcher"));
    job->addMetaData(QLatin1String("customHTTPHeader"), QLatin1String("Authorization: GoogleLogin auth="+ m_connection->authorization()));

    connect( job, SIGNAL( data( KIO::Job*, const QByteArray& ) ),
             this, SLOT( slotData( KIO::Job*, const QByteArray& ) ) );
    connect( job, SIGNAL( result( KJob* ) ), this, SLOT( slotResult( KJob* ) ) );
    job->start();
}

void GooglereaderFeedDeleteJob::slotResult( KJob *job )
{
    if ( job->error() ) {
        kWarning() << job->errorString();
        setError( CouldNotDeleteFeed );
        setErrorText( job->errorString() );
        emitResult();
        return;
    }
    emitResult();
}


GooglereaderFeedFetchJob::GooglereaderFeedFetchJob( GooglereaderConnection *connection, QObject *parent )
    : FeedFetchJob( parent )
{
    m_connection = connection;
}

void GooglereaderFeedFetchJob::setFeed( const Akonadi::Collection& feed )
{
    m_feed = feed;
}

Akonadi::Collection GooglereaderFeedFetchJob::feed() const
{
    return m_feed;
}

QList<Akonadi::Item> GooglereaderFeedFetchJob::items() const
{
    return m_items;
}

QString GooglereaderFeedFetchJob::errorString() const
{
    QString result;
    switch ( error() ) {
        case KJob::NoError:
            result = i18n( "No error." );
            break;
        case FeedFetchJob::CouldNotFetchFeed:
            result = i18n( "Couldn't fetch feed.\n%1", errorText() );
            break;
        default:
            result = i18n( "Unknown result code." );
            break;
    }
    return result;
}

void GooglereaderFeedFetchJob::start()
{

    connect(m_connection, SIGNAL(loggedIn()), this, SLOT(loggedIn()));
    m_connection->login();

}

void GooglereaderFeedFetchJob::loggedIn()
{
    m_items.clear();

    KIO::StoredTransferJob * const job = KIO::storedGet( m_feed.xmlUrl(), KIO::NoReload, KIO::HideProgressInfo );
    job->addMetaData(QLatin1String("UserAgent"), QLatin1String("Google-Reader Akonadi News fetcher"));
    job->addMetaData(QLatin1String("customHTTPHeader"), QLatin1String("Authorization: GoogleLogin auth="+ m_connection->authorization()));
    connect( job, SIGNAL( result( KJob* ) ), this, SLOT( slotResult( KJob* ) ) );
    job->start();
}

void GooglereaderFeedFetchJob::slotResult( KJob *job )
{
    const KIO::StoredTransferJob * const sjob = qobject_cast<const KIO::StoredTransferJob*>( job );
    Q_ASSERT( sjob );
    
    if ( job->error() ) {
        kWarning() << job->errorString();
        //setError( CouldNotDeleteFeed );
        setErrorText( job->errorString() );
        emitResult();
        return;
    }

    QByteArray data = sjob->data();
    //kWarning() << "got data" << data;

    QJson::Parser parser;
    bool ok;
    //kWarning() << "data " << data;
    QVariantMap result = parser.parse(data, &ok).toMap();
    //kWarning() << "result" << result;

    kWarning() << "ok?" << ok;

    foreach (QVariant item, result[QLatin1String("items")].toList()) {
      QVariantMap map = item.toMap();

      Akonadi::Item akonadiItem;
      KRss::RssItem rssItem;

      kWarning() << "content" << map[QLatin1String("summary")].toMap()[QLatin1String("content")].toString();

      QString content = map[QLatin1String("summary")].toMap()[QLatin1String("content")].toString();
      if ( content.length() == 0 )
      {
          content = map[QLatin1String("content")].toMap()[QLatin1String("content")].toString();
      }
      
      rssItem.setTitle( map[QLatin1String("title")].toString() );
      rssItem.setContent( content );
      rssItem.setDatePublished( KDateTime(QDateTime::fromTime_t(map[QLatin1String("published")].toUInt()), KDateTime::UTC) );
      rssItem.setDateUpdated( KDateTime(QDateTime::fromTime_t(map[QLatin1String("updated")].toUInt()), KDateTime::UTC) );
      rssItem.setLink( map[QLatin1String("origin")].toMap()[QLatin1String("htmlUrl")].toString() );
      rssItem.setGuid( map[QLatin1String("id")].toString() );
      rssItem.setGuidIsHash( false );
      rssItem.setSourceFeedId( m_feed.id() );
      rssItem.setHeadersLoaded( true );
      rssItem.setContentLoaded( true );

      // Authors
      QList<KRss::Person> authors;
      KRss::Person author;
      author.setName( map[QLatin1String("author")].toString() );
      rssItem.setAuthors( authors );

      // Categories
      QList<KRss::Category> categories;
      foreach (QVariant category, result[QLatin1String("categories")].toList()) {
          if ( !category.toString().startsWith(QLatin1String("user/")) )
          {
            KRss::Category c;
            c.setLabel( category.toString() );
            categories.append( c );
          }
      }
      rssItem.setCategories( categories );
      
      akonadiItem.setPayload< KRss::RssItem >( rssItem );
      akonadiItem.setRemoteId( map[QLatin1String("id")].toString() );
      akonadiItem.setMimeType( KRss::Item::mimeType() );

      //if( map[QLatin1String("categories")].toList().contains(QLatin1String("user/") + m_connection->userid() + QLatin1String("/state/com.google/fresh")) )
      //{
      //    akonadiItem.setFlag( KRss::RssItem::flagNew() );
      //}

      if( map[QLatin1String("categories")].toList().contains(QLatin1String("user/") + m_connection->userid() + QLatin1String("/state/com.google/read")) )
      {
          akonadiItem.setFlag( KRss::RssItem::flagRead() );
      }
      else
      {
          akonadiItem.setFlag( KRss::RssItem::flagNew() );
      }

      if( map[QLatin1String("categories")].toList().contains(QLatin1String("user/") + m_connection->userid() + QLatin1String("/state/com.google/starred")) )
      {
          akonadiItem.setFlag( KRss::RssItem::flagImportant() );
      }
      
      kWarning() << "id:" << map[QLatin1String("id")];
      m_items.append(akonadiItem);
    }
    
    //const GooglereaderTransferJob * const njob = qobject_cast<const GooglereaderTransferJob*>( job );
    //Q_ASSERT( njob );
    /*const DeleteSubscriptionResponse * const response = dynamic_cast<const DeleteSubscriptionResponse*>(
                                                                           njob->soapMessage() );
    Q_ASSERT( response );

    if ( response->statusCode() != 0 ) {
        setError( CouldNotDeleteFeed );
        setErrorText( i18n( "Response error code: %1").arg( response->statusCode() ) );
        kWarning() << errorText();
        emitResult();
        return;
    }
    */
    emitResult();
}

GooglereaderItemModifyJob::GooglereaderItemModifyJob( QObject *parent )
    : ItemModifyJob( parent )
{
}

void GooglereaderItemModifyJob::setItem( const Akonadi::Item& item )
{
    m_item = item;
}

Akonadi::Item GooglereaderItemModifyJob::item() const
{
    return m_item;
}

QString GooglereaderItemModifyJob::errorString() const
{
    QString result;
    switch ( error() ) {
        case KJob::NoError:
            result = i18n( "No error." );
            break;
        case GooglereaderItemModifyJob::CouldNotContactBackend:
            result = i18n( "Couldn't contact the Googlereader service.\n%1", errorText() );
            break;
        case GooglereaderItemModifyJob::CouldNotModifyItem:
            result = i18n( "Couldn't modify item\n%1", errorText() );
            break;
        default:
            result = i18n( "Unknown result code." );
            break;
    }
    return result;
}

void GooglereaderItemModifyJob::start()
{
    const QStringList ids = m_item.remoteId().split( QLatin1Char( separator ) );

    // TODO ...

}

void GooglereaderItemModifyJob::slotResult( KJob *job )
{
    if ( job->error() ) {
        kWarning() << job->errorString();
        setError( CouldNotModifyItem );
        setErrorText( job->errorString() );
        emitResult();
        return;
    }

    emitResult();
}

#include "googlereaderjobs.moc"
